@@ -3,6 +3,7 @@ from typing import *
 from cognite.client import utils
 from cognite.client._api_client import APIClient
 from cognite.client.data_classes import Database, DatabaseList, Row, RowList, Table, TableList
+from cognite.client.utils._auxiliary import local_import
 
 
 class RawAPI(APIClient):
@@ -272,7 +273,14 @@ class RawRowsAPI(APIClient):
         self._LIST_LIMIT = 10000
 
     def __call__(
-        self, db_name: str, table_name: str, chunk_size: int = None, limit: int = None
+        self,
+        db_name: str,
+        table_name: str,
+        chunk_size: int = None,
+        limit: int = None,
+        min_last_updated_time: int = None,
+        max_last_updated_time: int = None,
+        columns: List[str] = None,
     ) -> Generator[Union[Row, RowList], None, None]:
         """Iterate over rows.
 
@@ -283,12 +291,20 @@ class RawRowsAPI(APIClient):
             table_name (str): Name of the table to iterate over rows for
             chunk_size (int, optional): Number of rows to return in each chunk. Defaults to yielding one row a time.
             limit (int, optional): Maximum number of rows to return. Defaults to return all items.
+            min_last_updated_time (int): Rows must have been last updated after this time. ms since epoch.
+            max_last_updated_time (int): Rows must have been last updated before this time. ms since epoch.
+            columns (List[str]): List of column keys. Set to `None` for retrieving all, use [] to retrieve only row keys.
         """
         return self._list_generator(
             resource_path=utils._auxiliary.interpolate_and_url_encode(self._RESOURCE_PATH, db_name, table_name),
             chunk_size=chunk_size,
             method="GET",
             limit=limit,
+            filter={
+                "minLastUpdatedTime": min_last_updated_time,
+                "maxLastUpdatedTime": max_last_updated_time,
+                "columns": self._make_columns_param(columns),
+            },
         )
 
     def insert(
@@ -329,6 +345,35 @@ class RawRowsAPI(APIClient):
             task_unwrap_fn=lambda task: task["json"]["items"], task_list_element_unwrap_fn=lambda row: row.get("key")
         )
 
+    def insert_dataframe(self, db_name: str, table_name: str, dataframe: Any) -> None:
+        """`Insert pandas dataframe into a table <https://docs.cognite.com/api/v1/#operation/postRows>`_
+
+        Use index as rowkeys.
+
+        Args:
+            db_name (str): Name of the database.
+            table_name (str): Name of the table.
+            dataframe (pandas.DataFrame): The dataframe to insert. Index will be used as rowkeys.
+            ensure_parent (bool): Create database/table if they don't already exist.
+
+        Returns:
+            None
+
+        Examples:
+
+            Insert new rows into a table::
+
+                >>> import pandas as pd
+                >>> from cognite.client import CogniteClient
+                >>>
+                >>> c = CogniteClient()
+                >>> df = pd.DataFrame(data={"a": 1, "b": 2}, index=["r1", "r2", "r3"])
+                >>> res = c.raw.rows.insert_dataframe("db1", "table1", df)
+        """
+        df_dict = dataframe.to_dict(orient="index")
+        rows = [Row(key=key, columns=cols) for key, cols in df_dict.items()]
+        self.insert(db_name, table_name, rows)
+
     def _process_row_input(self, row: List[Union[List, Dict, Row]]):
         utils._auxiliary.assert_type(row, "row", [list, dict, Row])
         rows = []
@@ -452,21 +497,83 @@ class RawRowsAPI(APIClient):
                 >>> for row_list in c.raw.rows(db_name="db1", table_name="t1", chunk_size=2500):
                 ...     row_list # do something with the rows
         """
-        if columns is not None:
-            if not isinstance(columns, List):
-                raise ValueError("Expected a list for argument columns")
-            if len(columns) == 0:
-                columns = ","
-            else:
-                columns = ",".join([str(x) for x in columns])
+        if limit in {None, -1, float("inf")}:
+            cursors = self._get(
+                url_path=utils._auxiliary.interpolate_and_url_encode(
+                    "/raw/dbs/{}/tables/{}/cursors", db_name, table_name
+                ),
+                params={
+                    "minLastUpdatedTime": min_last_updated_time,
+                    "maxLastUpdatedTime": max_last_updated_time,
+                    "numberOfCursors": self._config.max_workers,
+                },
+            ).json()["items"]
+        else:
+            cursors = [None]
+        tasks = [
+            dict(
+                resource_path=utils._auxiliary.interpolate_and_url_encode(self._RESOURCE_PATH, db_name, table_name),
+                method="GET",
+                filter={
+                    "columns": self._make_columns_param(columns),
+                    "minLastUpdatedTime": min_last_updated_time,
+                    "maxLastUpdatedTime": max_last_updated_time,
+                },
+                limit=limit,
+                initial_cursor=cursor,
+            )
+            for cursor in cursors
+        ]
+        summary = utils._concurrency.execute_tasks_concurrently(self._list, tasks, max_workers=self._config.max_workers)
+        if summary.exceptions:
+            raise summary.exceptions[0]
+        return RowList(summary.joined_results())
 
-        return self._list(
-            resource_path=utils._auxiliary.interpolate_and_url_encode(self._RESOURCE_PATH, db_name, table_name),
-            limit=limit,
-            method="GET",
-            filter={
-                "minLastUpdatedTime": min_last_updated_time,
-                "maxLastUpdatedTime": max_last_updated_time,
-                "columns": columns,
-            },
-        )
+    def _make_columns_param(self, columns: List[str]) -> Optional[str]:
+        if columns is None:
+            return None
+        if not isinstance(columns, List):
+            raise ValueError("Expected a list for argument columns")
+        if len(columns) == 0:
+            columns = ","
+        else:
+            columns = ",".join([str(x) for x in columns])
+        return columns
+
+    def retrieve_dataframe(
+        self,
+        db_name: str,
+        table_name: str,
+        min_last_updated_time: int = None,
+        max_last_updated_time: int = None,
+        columns: List[str] = None,
+        limit: int = 25,
+    ) -> "pandas.DataFrame":
+        """`Retrieve rows in a table as a pandas dataframe. <https://docs.cognite.com/api/v1/#operation/getRows>`_
+
+        Rowkeys are used as the index.
+
+        Args:
+            db_name (str): Name of the database.
+            table_name (str): Name of the table.
+            min_last_updated_time (int): Rows must have been last updated after this time. ms since epoch.
+            max_last_updated_time (int): Rows must have been last updated before this time. ms since epoch.
+            columns (List[str]): List of column keys. Set to `None` for retrieving all, use [] to retrieve only row keys.
+            limit (int): The number of rows to retrieve. Defaults to 25. Set to -1, float("inf") or None to return all items.
+
+        Returns:
+            pandas.DataFrame: The requested rows in a pandas dataframe.
+
+        Examples:
+
+            Get dataframe::
+
+                >>> from cognite.client import CogniteClient
+                >>> c = CogniteClient()
+                >>> df = c.raw.rows.retrieve_dataframe("db1", "t1", limit=5)
+        """
+        pd = local_import("pandas")
+        rows = self.list(db_name, table_name, min_last_updated_time, max_last_updated_time, columns, limit)
+        idx = [r.key for r in rows]
+        cols = [r.columns for r in rows]
+        return pd.DataFrame(cols, index=idx)
